/*******************************************************************************
 * Copyright (c) 2003, 2010 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

// @(#) $Id:  $
package at.spardat.xma.mdl.paging;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Widget;

import at.spardat.enterprise.fmt.AParseException;
import at.spardat.enterprise.fmt.IFmt;
import at.spardat.xma.component.ComponentClient;
import at.spardat.xma.mdl.AttachmentExceptionClient;
import at.spardat.xma.mdl.ModelChangeEvent;
import at.spardat.xma.mdl.UIDelegateClient;
import at.spardat.xma.mdl.ValidationErrorClient;
import at.spardat.xma.mdl.WModel;
import at.spardat.xma.mdl.table.ITableWMClient;
import at.spardat.xma.page.EventAdapter;
import at.spardat.xma.page.PageClient;
/**
 * Implements the UI-related event handling and glues together the {@link PagingWMClient} and its
 * corresponding {@link PagingControlClient}.
 *
 * @author gub
 * @since 2.3.0
 */
public class PagingUIDelegateClient extends UIDelegateClient {
    /** the SWT control */
    private PagingControlClient control;
    /** the XMA model */
    private PagingWMClient model;
    /** true during updating SWT controls */
    private boolean updatingUI=false;
    /** true during updating the XMA model */
    private boolean updatingModel=false;
    /** enabled flag of the whole Composite of the control */
    private boolean enabled = true;

    /**
     * Create the PagingUIDelegateClient.
     * @param model the attached model
     */
    public PagingUIDelegateClient(PagingWMClient model) {
        this.model=model;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#createControl(Object)
     */
    public Object createControl(Object parent) {
        if(!(parent instanceof Composite)) throw new IllegalArgumentException("parent must be a composite");
        return new PagingControlClient((Composite)parent, model.getPage().getSession().getContext().getLocale(), PagingControlClient.DEFAULT, SWT.BORDER);
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#addListeners(Object,EventAdapter)
     */
    public void addListeners(Object control, EventAdapter adapter) {
        ((PagingControlClient)control).addSelecionListener(adapter);
        ((PagingControlClient)control).addModifyListener(adapter);
        ((PagingControlClient)control).addFocusListener(adapter);
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#attachUI(Object,Object)
     */
    public void attachUI(Object control, Object label) throws AttachmentExceptionClient {
        if (!(control instanceof PagingControlClient)) {
            throw new AttachmentExceptionClient ("cannot attach " + control.getClass().getName() + " to PagingModel");
        }
        this.control=(PagingControlClient) control;
        // attach this delegat to all controls of the pagingcontrol
        this.control.startW.setData(this);
        this.control.fastBackW.setData(this);
        this.control.backW.setData(this);
        this.control.reloadW.setData(this);
        this.control.jumpW.setData(this);
        this.control.nextW.setData(this);
        this.control.fastNextW.setData(this);
        this.control.endW.setData(this);
        this.control.customizeW.setData(this);
        this.control.exportW.setData(this);
        this.control.pageSizeW.setData(this);
        // move values to UI
        currentPage2UI();
        pageSizeList2UI();
        pageSize2UI();
        //if the control is already disabled, set this at this.enabled - otherwise do not change this.enabled
        //as in the Gen class the GUI designer disable property is set directly at the widget (only for some models)
        enabled = this.control.getEnabled()?enabled:false;
        this.control.setEnabled(enabled); //set the model's enabled state at the control
        updateErrorState();
        stateChanged();
        ((ITableWMClient)model.table).setExternalSorter(model);
    }

    /**
     * Show the index of the page containing the current offset in the jump combo.
     * Updates the choices of the jump combo.
     * Does nothing if SHOW_JUMP is not set.
     */
    private void currentPage2UI() {
        updatingUI = true;
        try {
            if((control.style&PagingControlClient.SHOW_JUMP)!=0) {
                String text = Integer.toString(model.getCurrentPage()+1);
                text = model.getJumpFmt().format(text);
                Point sel=null;
                if(text.equals(control.jumpW.getText())) {
                    sel=control.jumpW.getSelection();
                }
                List items = updateJumpList();
                control.jumpW.setText(text);
                if(items.contains(text)) {
                    control.jumpW.select(items.indexOf(text));
                }
                if(sel!=null) {
                    control.jumpW.setSelection(sel);
                }
            }
        } finally {
            updatingUI = false;
        }
    }

    /**
     * Show the pageSize in the pageSize combo.
     */
    private void pageSize2UI() {
        updatingUI = true;
        try {
            if((control.style&PagingControlClient.SHOW_PAGESIZE)!=0) {
                String text = Short.toString(model.getPageSize());
                text = model.getPageSizeFmt().format(text);
                control.pageSizeW.setText(text);
            }
        } finally {
            updatingUI = false;
        }
    }

    /**
     * Set the choice list of the pageSize combo.
     */
    void pageSizeList2UI() {
        updatingUI = true;
        try {
            if((control.style&PagingControlClient.SHOW_PAGESIZE)!=0) {
                control.pageSizeW.setItems(model.getPageSizeList());
            }
        } finally {
            updatingUI = false;
        }
    }

    /**
     * Enables/disables the buttons.
     * Updates the info label if SHOW_INFO is set.
     */
    private void stateChanged() {
        boolean hasNext=model.hasNext();
        int offset=model.getOffset();
        int resultSize=model.getResultSize();
        int tableSize=model.getTable().size();
        boolean isValid=!model.getPage().hasValidationError(control.pageSizeW);
        if((control.style&PagingControlClient.SHOW_INFO)!=0) {
            String text;
            Locale locale = model.getPage().getSession().getContext().getLocale();
            ResourceBundle messages = ResourceBundle.getBundle(PagingControlClient.resourceBundle,locale);
            if(resultSize==Integer.MAX_VALUE) {
                text = messages.getString("shortInfo");
            } else {
                text = messages.getString("info");
            }
            MessageFormat format = new MessageFormat(text, locale);
            if (tableSize >0) {
                control.info.setText(format.format(new Object[]{new Integer(offset+1),new Integer(offset+tableSize),new Integer(resultSize)}));
            } else {
                control.info.setText("");
            }

            format = new MessageFormat(messages.getString("fastBackWTip"),locale);
            control.fastBackW.setToolTipText(format.format(new Object[]{new Integer(model.getJumpSize())}));
            format = new MessageFormat(messages.getString("fastNextWTip"),locale);
            control.fastNextW.setToolTipText(format.format(new Object[]{new Integer(model.getJumpSize())}));
        }
        control.startW.setEnabled(isValid&&offset>0);
        control.fastBackW.setEnabled(isValid&&offset>0);
        control.backW.setEnabled(isValid&&offset>0);
        control.reloadW.setEnabled(isValid);
        control.jumpW.setEnabled(isValid&&(offset>0||hasNext));
        control.nextW.setEnabled(isValid&&hasNext);
        control.fastNextW.setEnabled(isValid&&hasNext);
        control.endW.setEnabled(isValid&&hasNext&&resultSize!=Integer.MAX_VALUE);
        control.customizeW.setEnabled(isValid);
        control.exportW.setEnabled(isValid);
    }

    /**
     * Updates the choices of the jump combo.
     */
    private List updateJumpList() {
        int page = model.getCurrentPage()+1;
        List pageList = new ArrayList(5);
        int additional=0;
        if(page-model.getJumpSize()>0) pageList.add(Integer.toString(page-model.getJumpSize()));
        else additional++;
        if(page-1>0) pageList.add(Integer.toString(page-1));
        else additional++;
        pageList.add(Integer.toString(page));
        int maxPage = (int) Math.ceil((float)model.getResultSize()/(float)model.getPageSize());
        if(page+1<=maxPage) pageList.add(Integer.toString(page+1));
        if(page+model.getJumpSize()<=maxPage) pageList.add(Integer.toString(page+model.getJumpSize()));
        String[] pageListArray=new String[pageList.size()];
        for(int i=0;i<pageList.size();i++) {
            pageListArray[i]=(String) pageList.get(i);
        }
        control.jumpW.setItems(pageListArray);
        return pageList;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#detachUI()
     */
    public void detachUI() {
        if(control!=null) {
            enabled=control.getEnabled();
            // clear error state
            model.getPage().clearValidationErrorImpl(control.jumpW);
            model.getPage().clearValidationErrorImpl(control.pageSizeW);
            model.getPage().clearError(control);
            model.getPage().getDialogPage().clearWarning(control);
            model.getPage().getDialogPage().clearInfo(control);
        }
        control = null;
        ((ITableWMClient)model.table).setExternalSorter(null);
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#getUIControl()
     */
    public Object getUIControl() {
        if (!isUIAttached()) throw new IllegalStateException ("not attached");
        return control;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#getUILabel()
     */
    public Object getUILabel() {
        return null;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#getWModel()
     */
    public WModel getWModel() {
        return model;
    }


    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#handleModelChangeEvent(ModelChangeEvent)
     */
    public void handleModelChangeEvent(ModelChangeEvent event) {
        if (!isUIAttached()) return;
        if(updatingModel) return;
        currentPage2UI();
        pageSize2UI();
        updateErrorState();
        stateChanged();
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#handleUIEvent(Object,int)
     */
    public void handleUIEvent(Object event, int type) {
        if (!isUIAttached()) return;
        if (updatingUI) return;
        try {
            updatingModel=true;
            if(event instanceof SelectionEvent) {
                Widget button = ((SelectionEvent)event).widget;
                if(button==control.startW) {
                    model.start();
                } else if(button==control.fastBackW) {
                    model.fastBack();
                } else if(button==control.backW) {
                    model.back();
                } else if(button==control.reloadW) {
                    model.reload();
                } else if(button==control.nextW) {
                    model.next();
                } else if(button==control.fastNextW) {
                    model.fastNext();
                } else if(button==control.endW) {
                    model.end();
                } else if(button==control.customizeW) {
                    model.customize();
                } else if(button==control.exportW) {
                    model.export();
                } else if(button instanceof Combo) {
                    handleComboChanged(button);
                }
            } else if(event instanceof ModifyEvent) {
                Widget combo = ((ModifyEvent)event).widget;
                if(combo instanceof Combo) {
                    handleComboChanged(combo);
                }
            }
        } finally {
            updatingModel=false;
            if(event instanceof SelectionEvent) {
                currentPage2UI();
            }
            updateErrorState();
            stateChanged();
        }
    }

    /**
     * Handles the UI-events concerning the two combos.
     */
    void handleComboChanged(Widget combo) {
        if(combo==control.jumpW) {
            String text = control.jumpW.getText();
            try {
                text = model.getJumpFmt().parse(text);
                if(text.length()==0) return;
                model.getPage().clearValidationErrorImpl(control.jumpW);
                int page=Integer.parseInt(text)-1;
                model.jump(page);
//                if(page!=model.getCurrentPage()) {
                    currentPage2UI();
//                }
            } catch (AParseException ape) {
                model.getPage().setValidationErrorImpl(new ValidationErrorClient (model, control.jumpW, "", ape));
            }
        } else if(combo==control.pageSizeW) {
            String text = control.pageSizeW.getText();
            try {
                text = model.getPageSizeFmt().parse(text);
                model.getPage().clearValidationErrorImpl(control.pageSizeW);
                short pageSizeFromCombo = 0;
                try {
                  pageSizeFromCombo  = Short.parseShort(text);
                } catch (NumberFormatException e){
                    throw new AParseException(e.getMessage());
                }
                boolean valueChanged = model.getPageSize() != pageSizeFromCombo;
                model.setPageSizeFromUI(pageSizeFromCombo);
                if (valueChanged) {
                    model.reload();                    
                }
            } catch (AParseException ape) {
                model.getPage().setValidationErrorImpl(new ValidationErrorClient (model, control.pageSizeW, control.pageSizeL.getText(), ape));
            }
        }
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#isEditable()
     */
    public boolean isEditable() {
        return true;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#isEnabled()
     */
    public boolean isEnabled() {
        if (isUIAttached()) {
            //always take the state from the widget (if it was set directly on it) as this is the old behavior
            enabled = control.isEnabled();
        }
        return enabled;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#isUIAttached()
     */
    public boolean isUIAttached() {
        return control!=null;
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#setEditable(boolean)
     */
    public void setEditable(boolean what) {
        // TODO Auto-generated method stub
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#setEnabled(boolean)
     */
    public void setEnabled(boolean what) {
        enabled = what;
        if (isUIAttached()){
            control.setEnabled(what);
        }
    }

    /* (non-Javadoc)
     * @see at.spardat.xma.mdl.UIDelegateClient#setErrorColor(boolean)
     */
    public void setErrorColor(boolean inError) {
        if(model.getPage().hasValidationError(control.pageSizeW)) {
            control.pageSizeW.setBackground (model.getPage().getComponent().getColor(ComponentClient.errorBackground));
        } else {
            control.pageSizeW.setBackground(null);
        }
        if(model.getPage().hasValidationError(control.jumpW)) {
            control.jumpW.setBackground (model.getPage().getComponent().getColor(ComponentClient.errorBackground));
        } else {
            control.jumpW.setBackground(null);
        }
    }

    /**
     * This method must be called after every change in an SWT Text control to update
     * the error state information (i.e., keep track of the validation errors in
     * entry fields).
     */
    void updateErrorState() {
        if((control.style&PagingControlClient.SHOW_PAGESIZE)!=0) {
            updateErrorState(control.pageSizeW,model.getPageSizeFmt(),control.pageSizeL.getText());
        }
        if((control.style&PagingControlClient.SHOW_JUMP)!=0) {
            updateErrorState(control.jumpW,model.getJumpFmt(),"");
        }
    }

    /**
     * Updates the error state for one given combo.
     */
    void updateErrorState(Combo combo, IFmt formatter, String labelText) {
        String text = combo.getText();
        if (formatter == null || formatter.isOneWay()) {
            // clear a previous error state
            model.getPage().clearValidationErrorImpl(combo);
            return;
        }
        // pass the text through the formatter to check if it is legitimate external
        // input for the widget model
        AParseException parseException = null;
        try {
            formatter.parse(text);
        } catch (AParseException ex) {
            parseException = ex;
        }
        PageClient pageModel = model.getPage();
        if (parseException != null) {
            // construct a ValidationErrorClient object and register it with the Page
            pageModel.setValidationErrorImpl(new ValidationErrorClient (model, combo, labelText, parseException));
        } else {
            // clear a previous error state
            pageModel.clearValidationErrorImpl(combo);
        }
    }

    boolean isValid() {
        if(!isUIAttached()) return true;
        return model.getPage().hasValidationError(control.pageSizeW) || model.getPage().hasValidationError(control.jumpW);
    }
}
